/*************************************************************************************************/
/* display_routines.c */
/*************************************************************************************************/

#ifndef cmh_f_cpu_
#define cmh_f_cpu_
#define F_CPU 8000000L
#endif

#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include "chicken_mobile.h"
#include "glcd_routines.h"
#include "keyboard_routines.h"
#include "display_routines.h"
#include "cmh_util.h"

#define RTC_ADDR 0x68

uint8_t ee_display_light_level;

/* display_startup -----------------------------------------------------------------------*
* shows initiation screen on the display 
*/
void display_startup(void) {
	extern display_status_t display_status;
	extern const char str_title[];
	extern const char timeP[], dateP[];
	
	set_display_light(display_status.light_level);
	glcd_clear_screen();
	_delay_ms(250);
	glcd_set_font(6);
	_delay_ms(10);
	glcd_set_cursor(GLCD_MAXX / 2, 24);
	glcd_draw_pgm_center_str(str_title);
	_delay_ms(250);
	glcd_draw_rectangle(20, 22, 191 - 20, 22 + 16, 1);
	glcd_set_font(0);
	_delay_ms(250);
	glcd_set_font(0);
	glcd_set_cursor(0, 54);
	glcd_draw_pgm_string(dateP);
	glcd_draw_char(' ');
	glcd_draw_pgm_string(timeP);
	_delay_ms(1000);
};

/* show_date -----------------------------------------------------------------------------*
* Displays the date at the upper left corner of the screen 
* Input: pointer to date-time array
*/
void show_date(datetime_t *date_time, const uint8_t x_pos, const uint8_t y_pos) {
	uint8_t month_list_pnt;
	char date_string[10];
	extern const char str_month_list[];
	
	glcd_set_cursor(x_pos, y_pos);
	date_string[ 0] = (date_time->day / 10) + '0';
	date_string[ 1] = (date_time->day % 10) + '0';
	date_string[ 2] = '-';
	month_list_pnt = (date_time->month-1) * 3;
	date_string[ 3] = pgm_read_byte(str_month_list + month_list_pnt);
	date_string[ 4] = pgm_read_byte(str_month_list + month_list_pnt + 1);
	date_string[ 5] = pgm_read_byte(str_month_list + month_list_pnt + 2);
	date_string[ 6] = '-';
	date_string[ 7] = (date_time->year / 10) + '0';
	date_string[ 8] = (date_time->year % 10) + '0';

	date_string[ 9] = 0;
	glcd_draw_string(date_string);
}

/* show_time -----------------------------------------------------------------------------*
* Displays the time at the upper right corner of the screen
* Input: pointer to date-time array
*/
void show_time(datetime_t *date_time, const uint8_t x_pos, const uint8_t y_pos) {
	char time_string[6];
	
	glcd_set_cursor(x_pos, y_pos);
	time_string[ 0] = (date_time->hour / 10) + '0';
	time_string[ 1] = (date_time->hour % 10) + '0';
	time_string[ 2] = ':';
	time_string[ 3] = (date_time->min / 10) + '0';
	time_string[ 4] = (date_time->min % 10) + '0';
	time_string[ 5] = 0;
	glcd_draw_string(time_string);
}

/* show_menu -----------------------------------------------------------------------------*
* Shows a menu with 4 items on the display
* Input: str_menu - pointer to program memory with 4 consecutive 0-terminated strings
*/
void show_menu(const char *menu_str) {
	extern datetime_t date_time;
	
	// title bar on top of the screen
	glcd_draw_filled_rectangle(0, 0, GLCD_MAXX - 1, 8, 0);
	glcd_draw_line(0, 9, GLCD_MAXX - 1, 9, 1);
	glcd_set_font(0);
	show_date(&date_time, DISPLAY_POS_DATE_X, 0);			// date in upper left corner
	show_time(&date_time, DISPLAY_POS_TIME_X, 0);			// time in upper right corner
	glcd_set_cursor(109, 0);
	glcd_draw_pgm_center_str(menu_str);
	while ((pgm_read_byte(menu_str)) > 0) ++menu_str;
	++menu_str;

	// menu bar at the bottom of the screen
	glcd_draw_filled_rectangle(0, 54, GLCD_MAXX - 1, GLCD_MAXY - 1, 0);
	glcd_draw_line(0, 54, GLCD_MAXX - 1, 54, 1);
	glcd_draw_line(GLCD_MAXX / 4, 55, GLCD_MAXX / 4, GLCD_MAXY - 1, 1);
	glcd_draw_line(2 * GLCD_MAXX / 4, 55, 2 * GLCD_MAXX / 4, GLCD_MAXY - 1, 1);
	glcd_draw_line(3 * GLCD_MAXX / 4, 55, 3 * GLCD_MAXX / 4, GLCD_MAXY - 1, 1);
	glcd_set_font(2);
	glcd_set_cursor(GLCD_MAXX / 8 , 56);
	glcd_draw_pgm_center_str(menu_str);
	while ((pgm_read_byte(menu_str)) > 0) ++menu_str;
	++menu_str;
	glcd_set_cursor(GLCD_MAXX  / 8 * 3, 56);
	glcd_draw_pgm_center_str( menu_str);
	while ((pgm_read_byte(menu_str)) > 0) ++menu_str;
	++menu_str;
	glcd_set_cursor(GLCD_MAXX  / 8 * 5, 56);
	glcd_draw_pgm_center_str(menu_str);
	while ((pgm_read_byte(menu_str)) > 0) ++menu_str;
	++menu_str;
	glcd_set_cursor(GLCD_MAXX  / 8 * 7, 56);
	glcd_draw_pgm_center_str(menu_str);
	glcd_set_font(0);
	//_delay_ms(25);
};

// Cockpit display functions ---------------------------------------------------------------

/* show_batt_status ----------------------------------------------------------------------*/
static void show_batt_status(void) {
	extern batt_status_t batt_status;
	extern uint8_t sec_counter;
	const uint8_t x_pos = 50;
	
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10);
	glcd_draw_dec(batt_status.vcc, 4, 2);
	glcd_draw_char('V');
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 18);
	if (batt_status.vcc_status == BATT_LEVEL_HIGH) glcd_draw_string("-> high");
	else if (batt_status.vcc_status == BATT_LEVEL_CRIT) glcd_draw_string("-> crit");
	else {
		if (sec_counter & 0b0000001) glcd_draw_string("-> low ");
		else glcd_draw_string("#######");
	};
}

/* show_amb_light_sensors ---------------------------------------------------------------------*/
static void show_amb_light_sensors(void) {
	extern amb_light_status_t amb_light_status;
	const uint8_t x_pos = 100;

	glcd_set_cursor( x_pos, GLCD_WORKSPACE_Y_START + 10);
	glcd_draw_dec(amb_light_status.sens1, 4, 0);
	glcd_draw_dec(amb_light_status.sens2, 4, 0);
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 9);
	glcd_draw_string("Avg ");
	if (amb_light_status.light_mov_avg > 1024)
		glcd_draw_string(" ---");
	else
		glcd_draw_dec(amb_light_status.light_mov_avg, 4, 0);
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 18);
	if (amb_light_status.sens_status == LIGHT_LEVEL_HIGH) glcd_draw_string("-> high");
	else glcd_draw_string("-> low ");
}

/* show_temp_status ----------------------------------------------------------------------*/
static void show_temp_status(void) {
	extern temp_status_t temp_status;
	// extern int16_t temp_buffer_ext[];
	// extern int16_t temp_buffer_int[];
	// extern uint8_t temp_buffer_pnt;
	// uint8_t temp_buffer_previous;
	extern const char str_voc_temp_missing[];
	const uint8_t x_pos = 0;
		
	// show current temperature data
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10);
	glcd_draw_char('E');
	if (temp_status.err_flag_ext > 0) glcd_draw_pgm_string(str_voc_temp_missing);
	else glcd_draw_sdec(temp_status.temp_ext, 3, 1);
	glcd_draw_char('C');
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 9);
	glcd_draw_char('I');
	if (temp_status.err_flag_int > 0) glcd_draw_pgm_string(str_voc_temp_missing);
	else glcd_draw_sdec(temp_status.temp_int, 3, 1);
	glcd_draw_char('C');

/* debug -----

	// show most recent average log data
	if (temp_buffer_pnt == 0)							// set pointer to last average calculation
		temp_buffer_previous = TEMP_BUFFER_SIZE - 1;
	else
		temp_buffer_previous = temp_buffer_pnt - 1;
	
	glcd_set_font(2);
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 18);
	glcd_draw_string("AvgE");
	if (temp_buffer_ext[temp_buffer_previous] >= MAX_TEMP) glcd_draw_pgm_string(str_voc_temp_missing);
	else glcd_draw_sdec(temp_buffer_ext[temp_buffer_previous], 3, 1);
	glcd_draw_char('C');
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 25);
	glcd_draw_string("AvgI");
	if (temp_buffer_int[temp_buffer_previous] >= MAX_TEMP) glcd_draw_pgm_string(str_voc_temp_missing);
	else glcd_draw_sdec( temp_buffer_int[temp_buffer_previous], 3, 1);
	glcd_draw_char('C');
	glcd_set_font(0);
*/
}

/* show_output_status --------------------------------------------------------------------*/
static void show_output_status(void) {
	const uint8_t x_pos = 153;

	show_LED_level(x_pos, GLCD_WORKSPACE_Y_START + 10);
	show_mp3_status(x_pos, GLCD_WORKSPACE_Y_START + 10 + 9);
	// show_mp3_song_nr(x_pos, GLCD_WORKSPACE_Y_START + 10 + 18);
	show_hatch_status(x_pos, GLCD_WORKSPACE_Y_START + 10 + 18);
}

/* show_time_window --------------------------------------------------------------------------*
* Shows 2 times (hours and minutes) separated by a dash
*/
static void show_time_window(const uint8_t tw1_hour, const uint8_t tw1_min, const uint8_t tw2_hour, const uint8_t tw2_min) {
	show_2dig_dec(tw1_hour);
	glcd_draw_char(':');
	show_2dig_dec(tw1_min);
	glcd_draw_char('-');
	show_2dig_dec(tw2_hour);
	glcd_draw_char(':');
	show_2dig_dec(tw2_min);
}

/* initiate_cockpit --------------------------------------------------------------------------*
*/
void initiate_cockpit(void) {
	extern const char str_menu_cockpit[];
	
	show_menu(str_menu_cockpit);
	glcd_draw_filled_rectangle(0, GLCD_WORKSPACE_Y_START - 1, GLCD_MAXX - 1, GLCD_WORKSPACE_Y_END, 0);
	glcd_set_cursor(0, GLCD_WORKSPACE_Y_START);
	glcd_draw_string("Temp");
	glcd_draw_line(46, GLCD_WORKSPACE_Y_START, 46, GLCD_WORKSPACE_Y_START + 41, 1);
	glcd_set_cursor(50, GLCD_WORKSPACE_Y_START);
	glcd_draw_string("Battery");
	glcd_draw_line(96, GLCD_WORKSPACE_Y_START, 96, GLCD_WORKSPACE_Y_START + 41, 1);
	glcd_set_cursor(100, GLCD_WORKSPACE_Y_START);
	glcd_draw_string("AmbLight");
	glcd_draw_line(149, GLCD_WORKSPACE_Y_START, 149, GLCD_WORKSPACE_Y_START + 41, 1);
	glcd_set_cursor(153, GLCD_WORKSPACE_Y_START);
	glcd_draw_string("Output");
}

/* refresh_cockpit -----------------------------------------------------------------------*/
void refresh_cockpit(void) {
	extern datetime_t date_time;
	extern display_status_t display_status;
	
	show_date(&date_time, DISPLAY_POS_DATE_X, 0);
	show_time(&date_time, DISPLAY_POS_TIME_X, 0);
	show_temp_status();
	show_batt_status();
	show_amb_light_sensors();
	show_output_status();
	
	// debug
	// glcd_set_font(1);
    // glcd_set_cursor(172, 48);
	// glcd_draw_dec(display_status.timeout_cnt, 5, 0);
	// glcd_set_font( 0);
}

/* show_LED_level ------------------------------------------------------------------------*
* Shows the current LED level on display
*/
void show_LED_level(const uint8_t x_pos, const uint8_t y_pos) {
	extern LED_status_t LED_status;
	
	glcd_set_cursor(x_pos, y_pos);
	glcd_draw_string("LED");
	glcd_draw_dec(LED_status.led_level, 3, 0);
}

/* show MP3 status -------------------------------------------------------------------------
* Shows the MP3 status
*/
void show_mp3_status(const uint8_t x_pos, const uint8_t y_pos) {
	extern mp3_status_t mp3_status;

	glcd_set_cursor(x_pos, y_pos);
	glcd_draw_string("MP3 ");
	glcd_set_font(2);
	glcd_set_cursor(x_pos + 22, y_pos + 1);
	if (mp3_status.mp3_status == MP3_ON)
		glcd_draw_string("on ");
	else
		glcd_draw_string("off");
	glcd_set_font(0);
}

/* show MP3 song_nr -------------------------------------------------------------------------
* Shows the MP3 song status
*/
void show_mp3_song_nr(const uint8_t x_pos, const uint8_t y_pos) {
	extern mp3_status_t mp3_status;

	glcd_set_cursor(x_pos, y_pos);
	glcd_draw_string("Song");
	glcd_draw_dec(mp3_status.song_nr, 2, 0);
}

/* show_hatch_status ------------------------------------------------------------------------
* Shows the status of hatch control
*/
void show_hatch_status(const uint8_t x_pos, const uint8_t y_pos) {
	extern hatch_control_t hatch_control; 

	glcd_set_cursor(x_pos, y_pos);
	glcd_draw_string("Hat");
	if (hatch_control.hatch_cnt == 0) {
		glcd_set_font(2);
		glcd_draw_filled_rectangle(x_pos + 18, y_pos, x_pos + 18 + 18, y_pos + 8, 0);
		glcd_set_cursor(x_pos + 22, y_pos + 1);
		glcd_draw_string("off");
		glcd_set_font(0);
	} else {
		glcd_draw_dec(hatch_control.hatch_cnt, 3, 0);
	};
}

/* show_datetime_parms ---------------------------------------------------------------------
* Sows date and time as part of the edit menu
*/
void show_datetime_parms(uint8_t x_pos) {
	extern datetime_t date_time;
	extern const char str_show_parms_date_time[];
		
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START);
	glcd_draw_pgm_string(str_show_parms_date_time);
	show_date(&date_time, x_pos, GLCD_WORKSPACE_Y_START + 10);
	show_time(&date_time, x_pos, GLCD_WORKSPACE_Y_START + 10 + 9);
}

/* show_display_parms ----------------------------------------------------------------------
* Sows the display parameters
*/
void show_display_parms(uint8_t x_pos) {
	extern display_status_t display_status;
	extern const char str_show_parms_display[];
	
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START);
	glcd_draw_pgm_string(str_show_parms_display);
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10);
	glcd_draw_string("Back Light ");
	glcd_draw_dec(display_status.light_level, 2, 0);
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 9);
	glcd_draw_string("Time (min) ");
	glcd_draw_dec(display_status.timeout / 60, 2, 0);
}

/* show_batt_parms -----------------------------------------------------------------------
* Shows the battery parameters
*/
void show_batt_parms(uint8_t x_pos) {
	extern batt_status_t batt_status;
	extern const char str_show_parms_batt_threshold[];
		
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START);
	glcd_draw_pgm_string(str_show_parms_batt_threshold);
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10);
	glcd_draw_string( "High  >");
	glcd_draw_dec(batt_status.threshold_high, 4, 2);
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 9);
	glcd_draw_string("Crit. <");
	glcd_draw_dec(batt_status.threshold_crit, 4, 2);
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 18);
	glcd_draw_string("Low   <");
	glcd_draw_dec(batt_status.threshold_low, 4, 2);
}

/* show_LED_parms --------------------------------------------------------------------------
* Shows the parameters for LED control
*/
void show_LED_parms(uint8_t x_pos) {
	extern LED_status_t LED_status;
	extern const char str_show_parms_led_control[];
	
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START);
	glcd_draw_pgm_string(str_show_parms_led_control);
	
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10);
	glcd_draw_string("Time Window");
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 9);
	show_time_window(LED_status.led_tw_on_hour, LED_status.led_tw_on_min, LED_status.led_tw_off_hour, LED_status.led_tw_off_min);	
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 18);
	glcd_draw_string("Dur. ");
	glcd_draw_dec(LED_status.led_duration, 3, 0);
	glcd_draw_string("  Max ");
	glcd_draw_dec(LED_status.led_max, 2, 0);
}

/* show_amb_light_parms --------------------------------------------------------------------
* Shows the parameters for the ambient light sensors
*/
void show_amb_light_parms(uint8_t x_pos) {
	extern amb_light_status_t amb_light_status;
	extern const char str_show_parms_amb_light[];
	
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START);
	glcd_draw_pgm_string(str_show_parms_amb_light);
	
	glcd_set_cursor( x_pos, GLCD_WORKSPACE_Y_START + 10);
	glcd_draw_string("Thr ");
	glcd_draw_dec(amb_light_status.light_threshold, 4, 0);
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 9);
	glcd_draw_string("AvgDur");
	glcd_draw_dec(amb_light_status.light_mov_avg_dur, 2, 0);
}

/* show_mp3_parms --------------------------------------------------------------------------
* Shows the parameters for MP3 control
*/
void show_mp3_parms(uint8_t x_pos) {
	extern mp3_status_t mp3_status;
	extern const char str_show_parms_mp3[];
	extern const char str_voc_duration[];
	
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START);
	glcd_draw_pgm_string(str_show_parms_mp3);

	// time window 1
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10);
	glcd_draw_string("Time W1 ");
	show_time_window(mp3_status.mp3_tw1_on_hour, mp3_status.mp3_tw1_on_min, mp3_status.mp3_tw1_off_hour, mp3_status.mp3_tw1_off_min);

	// time window 2
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 9);
	glcd_draw_string("Time W2 ");
	show_time_window(mp3_status.mp3_tw2_on_hour, mp3_status.mp3_tw2_on_min, mp3_status.mp3_tw2_off_hour, mp3_status.mp3_tw2_off_min);
	
	// duration and songs
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 18);
	glcd_draw_pgm_string(str_voc_duration);
	glcd_draw_dec(mp3_status.mp3_duration, 2, 0);
}

/* show_hatch_control_parms --------------------------------------------------------------
* Shows the parameters for hatch control
*/
void show_hatch_control_parms(uint8_t x_pos) {
	extern hatch_control_t hatch_control;
	extern const char str_show_parms_hatch[];
	extern const char str_voc_duration[];

	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START);
	glcd_draw_pgm_string(str_show_parms_hatch);
	
	// show time 
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10);
	glcd_draw_string("Time   ");
	show_2dig_dec(hatch_control.hatch_hour);
	glcd_draw_char(':');
	show_2dig_dec(hatch_control.hatch_min);

	// show duration
	glcd_set_cursor(x_pos, GLCD_WORKSPACE_Y_START + 10 + 9);
	glcd_draw_pgm_string(str_voc_duration);
	glcd_draw_dec(hatch_control.hatch_dur, 3, 0);
}

/* message_box -------------------------------------------------------------------------*
* Shows a message
*/
void message_box(const char *msg, const uint8_t delay_sec) {
	uint8_t i;
	
	glcd_draw_filled_rectangle(MESSAGE_BOX_X + 1, MESSAGE_BOX_Y + 1, GLCD_MAXX - MESSAGE_BOX_X - 1, GLCD_MAXY - MESSAGE_BOX_Y - 1, 0);
	glcd_draw_rectangle(MESSAGE_BOX_X, MESSAGE_BOX_Y, GLCD_MAXX - MESSAGE_BOX_X, GLCD_MAXY - MESSAGE_BOX_Y, 1);
	glcd_set_cursor(GLCD_MAXX / 2, GLCD_MAXY / 2 - 4);
	glcd_draw_pgm_center_str(msg);
	
	_delay_us(100);
	for (i = 0; i < delay_sec; ++i) {
		_delay_ms(1000);
	};
}

/* draw_timeline ------------------------------------------------------------------------
* Draws the time line for temperature log display
* Local function, called by show_temp_log()
*/

static void draw_timeline(void) {
	extern datetime_t date_time;
	uint8_t hour = date_time.hour, min = date_time.min;
	uint8_t i;
	
	glcd_set_font(1);
	min = min / 10 * 10;							// round down to 10 minutes
	for (i = TEMP_BUFFER_SIZE; i > 0; --i) {						
		if (min == 0) {
			glcd_draw_line(i - 1, 61, i - 1, 63, 1);
			if (((hour % 4) == 0) && (i > 4)) {
				if ((hour == 0) || (hour > 9)) {
					glcd_set_cursor(i - 8, 55);
					glcd_draw_char(' ');
					if (hour == 0) {
						glcd_draw_char('2');
						glcd_draw_char('4');
						glcd_draw_dotted_ver_line(i - 1, 0, 53, 1, 1);
					} else {
						glcd_draw_char(hour / 10 + '0');
						glcd_draw_char(hour % 10 + '0');
					};
				} else {
					glcd_set_cursor(i - 6, 55);
					glcd_draw_char(' ');
					glcd_draw_char(hour + '0');
				};
				glcd_draw_char(' ');
			};
			min = 50;
			if (hour == 0) hour = 23;
			else --hour;
		} else {
			min -= 10;
		};
	};
	glcd_set_font(0);
} 


/* draw_tickmark_label
*/
void draw_tickmark_label(int8_t my_label) {
	if (my_label < 0) {
			glcd_draw_char('-');
			my_label = -my_label;
		} else {
			glcd_draw_char(' ');
		};
	if (my_label > 9) glcd_draw_char(my_label / 10 + '0');
	glcd_draw_char(my_label % 10 + '0');
}

/* draw_temp_axis -----------------------------------------------------------------------
*
*/
static void draw_temp_axis(int16_t scale_min, int16_t scale_max) {
	int16_t my_temp = scale_max, range = scale_max - scale_min;
	uint8_t y;	
	
	glcd_set_font(1);
	glcd_draw_line(TEMP_BUFFER_SIZE, 0, TEMP_BUFFER_SIZE, 63, 1);

	// plot vertical y-axis
	glcd_draw_line(TEMP_BUFFER_SIZE + 1, 0, TEMP_BUFFER_SIZE + 3, 0, 1);
	
	// plot top label
	glcd_set_cursor(TEMP_BUFFER_SIZE + 5, 0);
	draw_tickmark_label(my_temp / 10);

	// plot lower labels with step width of 5.0 
	while (my_temp > scale_min) {
		my_temp -= 50;
		y = 64 - (my_temp - scale_min) * 64 / range;
		if (y > 53) y = 63;
		glcd_draw_line(TEMP_BUFFER_SIZE + 1, y, TEMP_BUFFER_SIZE + 3, y, 1);
		if (y < 3) {
			glcd_set_cursor(TEMP_BUFFER_SIZE + 4, 0);
		} else if (y > 61) {
			glcd_set_cursor(TEMP_BUFFER_SIZE + 4, 63 - 5);
		} else {
			glcd_set_cursor(TEMP_BUFFER_SIZE + 4, y - 2);
		};
		draw_tickmark_label(my_temp / 10);
	};
	glcd_set_font(0);
	
	// plot tick marks
	my_temp = scale_max;
	while (my_temp > scale_min) {
		y = 64 - (my_temp - scale_min) * 64 / range;
		glcd_set_pixel(TEMP_BUFFER_SIZE + 1, y, 1);
		glcd_set_pixel(TEMP_BUFFER_SIZE + 2, y, 1);
		if (range <= 150) my_temp -= 10;
		else my_temp -= 50;
	};
	glcd_set_pixel(TEMP_BUFFER_SIZE + 1, 63, 1);
	glcd_set_pixel(TEMP_BUFFER_SIZE + 2, 63, 1);
	
	// plot 0 degree line
	if ((scale_min < 0) && (scale_max > 0)) {
		y = 64 + scale_min * 64 / range;
		glcd_draw_dotted_hor_line(y, 0, TEMP_BUFFER_SIZE - 1, 3, 1);
	};
};
	
/* show_temp_log ------------------------------------------------------------------------
* Display of the external temperature log
*/
void show_temp_log(const char title[], int16_t temp_buffer[]) {
	extern uint8_t temp_buffer_pnt;
	int16_t temp_min, temp_max, scale_min, scale_max, range; 
	uint8_t pnt, x_pos, y, y_prev = 100;
		
	glcd_clear_screen();
	glcd_set_cursor(0, 0);
	glcd_draw_pgm_string(title);
	draw_timeline();
	
	temp_min = MAX_TEMP;
	temp_max = -MAX_TEMP;
	for (int i = 0; i < TEMP_BUFFER_SIZE; ++i) {
		if (temp_buffer[i] != MAX_TEMP) {
			if (temp_buffer[i] < temp_min) temp_min = temp_buffer[i];
			if (temp_buffer[i] > temp_max) temp_max = temp_buffer[i];
		};
	};
	glcd_set_cursor(0, 10);
	if (temp_min == MAX_TEMP) {
		glcd_draw_string("No data");
		return;
	};
	
	// show minimum and maximum
	glcd_set_cursor(0, 10);
	glcd_draw_string("Max");
	glcd_draw_sdec(temp_max, 3, 1);
	glcd_draw_char('C');
	glcd_set_cursor(0, 19);
	glcd_draw_string("Min");
	glcd_draw_sdec(temp_min, 3, 1);
	glcd_draw_char('C');
	_delay_ms(1);
	
	// draw y-scale
	scale_min = temp_min / 50 * 50;
	if (temp_min < 0) scale_min -= 50;
	scale_max = temp_max / 50 * 50;
	if (temp_max > 0) scale_max += 50;
	draw_temp_axis(scale_min, scale_max);
	range = scale_max - scale_min;
	
	// draw curve
	x_pos = TEMP_BUFFER_SIZE;
	pnt = temp_buffer_pnt;
	for (int i = 0; i < TEMP_BUFFER_SIZE; ++i) {
		if (pnt == 0) pnt = TEMP_BUFFER_SIZE - 1;
		else --pnt;
		--x_pos;
		if (temp_buffer[pnt] < MAX_TEMP) {
			y = 64 - (temp_buffer[pnt] - scale_min) * 64 / (range);
			if (y_prev >= 100) {
				glcd_set_pixel(x_pos, y, 1);
			} else {
				glcd_draw_line(x_pos, y_prev, x_pos, y, 1);
			};
			y_prev = y;
		} else {
			y_prev = 100;
		};
		_delay_us(100);
	}
}

/* show_temp_log_ext 
*
*/
void show_temp_log_ext(void) {
	extern 	int16_t temp_buffer_ext[];
	extern const char tit_temp_log_ext[];
	
	show_temp_log(tit_temp_log_ext, temp_buffer_ext);
}

/* show_temp_log_int --------------------------------------------------------------------
* Display of the internal temperature log
*/
void show_temp_log_int(void) {
	extern 	int16_t temp_buffer_int[];
	extern const char tit_temp_log_int[];
		
	show_temp_log(tit_temp_log_int, temp_buffer_int);
}

/* show_batt_log ------------------------------------------------------------------------
* Display of the battery voltage log 
*/
void show_batt_log(void) {
	extern uint16_t battery_buffer[];
	extern uint8_t temp_buffer_pnt;
	extern batt_status_t batt_status;
	extern const char tit_batt_log[];
	uint16_t batt_min = MAX_BATT, batt_max = 0;
	uint8_t y, my_batt, x_pos, pnt, y_prev = 100;
	const uint16_t scale_min = 1050, scale_max = 1450, range = 1450 - 1050;
	
	glcd_clear_screen();
	glcd_set_cursor(0, 0);
	glcd_draw_pgm_string(tit_batt_log);
	
	// draw x axis
	draw_timeline();
	
	for (int i = 0; i < TEMP_BUFFER_SIZE; ++i) {
		if (battery_buffer[i] < MAX_BATT) {
			if (battery_buffer[i] < batt_min) batt_min = battery_buffer[i];
			if (battery_buffer[i] > batt_max) batt_max = battery_buffer[i];
		};
	};

	// show minimum and maximum
	glcd_set_cursor(0, 10);
	if (batt_min == MAX_BATT) {
		glcd_draw_string("No data");
	} else {
		glcd_draw_string("Max");
		glcd_draw_sdec(batt_max, 4, 2);
		glcd_draw_char('V');
		glcd_set_cursor(0, 19);
		glcd_draw_string("Min");
		glcd_draw_sdec(batt_min, 4, 2);
		glcd_draw_char('V');
	};
	_delay_us(100);
	
	// draw marker line at low and critical voltages
	y = 64 - (batt_status.threshold_high - scale_min) * 64 / range;
	glcd_draw_dotted_hor_line(y, 0, TEMP_BUFFER_SIZE - 1, 3, 1);
	y = 64 - (batt_status.threshold_crit - scale_min) * 64 / range;
	glcd_draw_dotted_hor_line(y, 0, TEMP_BUFFER_SIZE - 1, 3, 1);
	y = 64 - (batt_status.threshold_low - scale_min) * 64 / range;
	glcd_draw_dotted_hor_line(y, 0, TEMP_BUFFER_SIZE - 1, 3, 1);

	// draw y-scale 
	glcd_draw_line(TEMP_BUFFER_SIZE, 0, TEMP_BUFFER_SIZE, 63, 1);
	glcd_set_font(1);
	my_batt = scale_max / 10;
	for (y = 0; y < 64; y += 8) {
		glcd_set_pixel(TEMP_BUFFER_SIZE + 1, y, 1);
		glcd_set_pixel(TEMP_BUFFER_SIZE + 2, y, 1);
		if ((my_batt % 10) == 0) {
			glcd_set_pixel(TEMP_BUFFER_SIZE + 3, y, 1);
			glcd_set_pixel(TEMP_BUFFER_SIZE + 4, y, 1);
			glcd_set_cursor(TEMP_BUFFER_SIZE + 7, y - 2);
			glcd_draw_dec(my_batt / 10, 2, 0);
		};
		my_batt -= 5;
	};
	glcd_set_font(0);
	
	// draw curve
	x_pos = TEMP_BUFFER_SIZE;
	pnt = temp_buffer_pnt;
	for (int i = 0; i < TEMP_BUFFER_SIZE; ++i) {
		if (pnt == 0) 
			pnt = TEMP_BUFFER_SIZE - 1;
		else
			--pnt;
		--x_pos;
		if (battery_buffer[pnt] < MAX_BATT) {
			y = 64 - (battery_buffer[pnt] - scale_min) * 64 / range;
			if (y_prev >= 100) {
				glcd_set_pixel(x_pos, y, 1);
			} else {
				glcd_draw_line(x_pos, y_prev, x_pos, y, 1);
			};
			y_prev = y;
		} else {
			y_prev = 100;
		};
	};
	_delay_us(100);
}

/* show_light_start_time ---------------------------------------------------------------------------*/
void show_light_start_time(void) {
	extern light_start_time_t light_start_time[];
	extern uint8_t light_start_pnt;
	extern datetime_t date_time;
	extern const char tit_start_time_log[];
	uint8_t my_day = date_time.day, my_month = date_time.month;
	uint8_t my_pnt = light_start_pnt;
		
	glcd_clear_screen();
	glcd_set_cursor(0, 0);
	glcd_draw_pgm_string(tit_start_time_log);
	glcd_draw_line(0, 9, GLCD_MAXX, 9, 1);
	glcd_draw_line(GLCD_MAXX / 2, 10, GLCD_MAXX / 2, GLCD_MAXY, 1);
	for(uint8_t i = 0; i < LIGHT_START_BUFFER_SIZE; ++i) {
		glcd_set_cursor((GLCD_MAXX / 2) * (i / 5) + 5, 13 + (i % 5) * 10);
		glcd_draw_dec(my_day, 2, 0);
		glcd_draw_char('.');
		glcd_draw_dec(my_month, 2, 0);
		glcd_draw_string(".: ");
		if (light_start_time[my_pnt].hour > 24) {
			glcd_draw_string("--:--");
		} else {
			show_2dig_dec(light_start_time[my_pnt].hour);
			glcd_draw_char(':');
			show_2dig_dec(light_start_time[my_pnt].min);
		};
		if (my_pnt == 0) my_pnt = LIGHT_START_BUFFER_SIZE - 1;
		else --my_pnt;
		decrease_day(&my_day, &my_month);
	};
}


